package ZipTie::Adapters::Brocade::vRouter::Disconnect;    

use strict;
use warnings;
use 5.010_000;

use ZipTie::CLIProtocol;
use ZipTie::Response;
use ZipTie::Logger;

use Exporter qw(import);
our @EXPORT_OK = qw(disconnect);

# Get the instance of the ZipTie::Logger module
my $LOGGER = ZipTie::Logger::get_logger();

sub disconnect {
	# Grab the CLI protocol
	my $cliProtocol = shift;
	
	# Send the Control-C command which is 0x03 in hexidecimal
	$cliProtocol->send_as_bytes("03");
	
	# Keep sending the "exit" command until we have successfully exited the device
	_send_exit($cliProtocol);
	
	# Disconnect from the CLI protocol
	$cliProtocol->disconnect();
}

sub _send_exit {
	# Grab the CLI protocol
	my $cliProtocol = shift;
	
	# Specify the responses to handle
	my @responses = ();
	push(@responses, ZipTie::Response->new("#|>", \&_send_exit));
	push(@responses, ZipTie::Response->new(".*"));
	
	# Send the "exit" command
	$cliProtocol->send("exit");
	my $response = $cliProtocol->wait_for_responses(\@responses);
	
	# Based on the response of the device, determine the next interaction that should be executed.
	my $nextInteraction = undef;
	if ($response)
	{
		$nextInteraction = $response->get_next_interaction();
	} else {
		$LOGGER->fatal("Invalid response from device encountered!");
	}
	
	# Call the next interaction if there is one to call
	if ($nextInteraction) {
		return &$nextInteraction($cliProtocol);
	}
}

1;
