#!/usr/bin/perl

use strict;
use warnings;

use English qw( -no_match_vars );
# Friendly names for punctuation variables, like $@ to $EVAL_ERROR

use ZipTie::Adapters::PaloAlto::PANOS::AutoLogin;
use ZipTie::Adapters::PaloAlto::PANOS::ParsingUtils qw(get_value_from_xml);
use ZipTie::Adapters::Utils qw( get_crep );
use ZipTie::Client;
use ZipTie::Discovery qw( get_connection_path get_devices add_or_update_devices );
use ZipTie::Logger;

my $ADAPTER = 'ZipTie::Adapters::PaloAlto::PANOS';

my $LOGGER = ZipTie::Logger::get_logger();

execute_discovery( $ARGV[0] );

sub execute_discovery {
	my ($management_server_ip) = @_;

	# Enable logging to file
	if ( $ENV{ADAPTER_LOG_TO_FILE} ) {
		$LOGGER->enable_logging_to_file( 'PANOS', 'PanoramaDiscovery', $management_server_ip );
	}

        # Check parameter is a valid ip
        my $ip_regex = get_crep('cipm');
        if($management_server_ip !~ /^$ip_regex$/){
                _fatal("Invalid IP address: $management_server_ip");
        }

	# Obtain a connection to the Ziptie API
	my $ziptie_client = ZipTie::Client->new();

	# Create a connection path object to use when connecting to the device
	my $connection_path = get_connection_path( $ziptie_client, $management_server_ip );

	#my $test = $connection_path->to_xml_string(1);
	#$LOGGER->debug("==================\n\n$test\n\n==================\n\n");

	#Log in to the device
	my $connection = ZipTie::Adapters::PaloAlto::PANOS::AutoLogin->new($connection_path);
	$connection->establish_connection();

	# Issue the command
	my $eval_success = 1;
	my $show_device_connected;
	$eval_success = eval {
		$show_device_connected = $connection->issue_command( '<show><devices><connected></connected></devices></show>', 'show devices connected' );
		1;
	};
	if ( !$eval_success ) {
		print "ERROR: Failed to issue 'show devices connected' command: $EVAL_ERROR\n";
		$LOGGER->fatal("Failed to issue 'show devices connected' command: $EVAL_ERROR");
	}

	# Parse the IP addresses of the managed devices from the output of the show device connected command
	my @device_ips;
	my @devices = $show_device_connected->findnodes('/response/result/devices/entry');
        for my $device ( @devices ) { push @device_ips, get_value_from_xml( 'ip-address', $device ); }

	my $existing_devices = get_devices( $ziptie_client, $management_server_ip );

	add_or_update_devices( $ziptie_client, $ADAPTER, $management_server_ip, $existing_devices, \@device_ips );

	$ziptie_client->logout();

	return;
}
